<!doctype html>
<html>
  <head>
    <title>Internal Server Error</title>
  </head>
  <body>

    <h1>Internal Server Error</h1>

    <p>='(</p>

    <% if @sentry_event_id %>
      <script>
      Raven.showReportDialog({
        // grab the eventId generated by the Sentry SDK
        eventId: '<%= @sentry_event_id -%>'
      });
      </script>
    <% end %>

  </body>
</html>
